#!/usr/bin/python

#-----------------------------------------------------------------------
#
# Creates the demo infoServ IOC data files from the data of an actual IOC
#
# requires:
#
#   Python 2.7.x
#
#   BeautifySoup4 python library:
#     http://www.crummy.com/software/BeautifulSoup/
#
# usage:
#
#   python makeDemo.py <top-level infoServ URL>
#
#   for example:
#     python makeDemo.py http://howler1:7729
#
#-----------------------------------------------------------------------

#
# import standard python modules we'll need
#
import os
import sys
import glob
import urllib2

#
# try to import Beautiful Soup 4 module
#
try:
  import bs4
except Exception,e:
  print "EXCEPTION: %s" % (str(e))
  print "(is BeautifulSoup 4 installed correctly?)"
  quit(1)

#-----------------------------------------------------------------------
def getContentFromURL(url,timeoutsec=10):
  #
  # return the data content from a given URL
  #
  try:
    requrl = urllib2.Request(url)
    usock = urllib2.urlopen(requrl,timeout=timeoutsec)
    udata = usock.read()
    usock.close()
  except Exception, e:
    return False, str(e)
  return True, udata

#-----------------------------------------------------------------------
def processURL(baseUrl,thisUrl):
  #
  # get content data
  #
  print "fetching data from '%s'" % (thisUrl)
  okay, result = getContentFromURL(thisUrl)
  if not okay:
    print "ERROR: %s" % (result)
    return False, None
  urlData = result

  #
  # construct base filename we'll save data to
  #
  filePart = thisUrl
  if filePart.startswith(baseUrl): filePart = filePart[len(baseUrl):]
  frList = [ ( "http://", ""  ),
             ( "/",       ""  ),
             ( ".",       "_" ),
             ( ",",       "-" ),
             ( "=",       "-" ),
             ( "?",       ""  ) ]
  for fnd,rpl in frList:
    filePart = filePart.replace(fnd,rpl)
  if filePart == "":
    filePart = "demo_top"
  else:
    filePart = "demo_" + filePart


  #
  # need to determine if content is html or something else (it'd be text)
  #
  if (urlData.lower().find("<body>") < 0):
    #
    # this is not an HTML file, save data as-is to a text file
    #
    fileName = filePart + ".txt"
    saveData = urlData
  else:
    #
    # this looks like an HTML file
    #
    # examine data with Beautiful Soup looking for URL references
    # we'll need to make recursive calls for any found
    # and convert them to a file references in our data
    #
    soup = bs4.BeautifulSoup(urlData)
    fileName = filePart + ".html"
    linksList = soup.find_all('a')
    for link in linksList:
      # get href target, skip link if it doesn't have one
      href = link.get("href")
      if href is None: continue
      # construct link's full URL
      if not href.lower().startswith("http"):
        linkUrl = baseUrl + href
      else:
        linkUrl = href
      okay, linkFile = processURL(baseUrl,linkUrl)
      if not okay:
        return False, None
      link["href"] = linkFile
    saveData = str(soup)

  #
  # if no output file was specified, create name for one
  #

  #
  # save data to output file
  #
  try:
    f = open(fileName,"wb")
  except Exception,e:
    print "EXCEPTION: %s" % (str(e))
    return False, None
  f.write(saveData)
  f.close()

  #
  # we're done with this URL
  #
  print "created '%s'" % (fileName)
  return True, fileName

#-----------------------------------------------------------------------
# main
#-----------------------------------------------------------------------

#
# first and only argument must be url of IOC's infoServ service
#
if len(sys.argv) != 2:
  print "Usage: python %s <top-level infoServ URL>" % (sys.argv[0])
  quit(1)
topUrl = sys.argv[1]

#
# start by deleting all demo_*.html/txt files
#
# we don't want any old ones lying around confusing things if the
# URLs available from the IOC change over time
#
deleteList = glob.glob("demo_*.html") + glob.glob("demo_*.txt")
for fileName in deleteList:
  print "deleting '%s'" % (fileName)
  os.remove(fileName)

#
# start recursive process to
#   pull down URL content
#   locate any URLs in it and change them to file references
#   save content to a local demo_*.html file
#   process any URLs previously discovered the same way
#
if not processURL(topUrl,topUrl):
  # processing failed
  # error messages will already have been printed, so we just bomb
  quit(1)

#
# that's it
#
print "done."
quit(0)
